// Unit:       LaunchM
// Version:    1.00
// Purpose:    implement context menu to Launch KEdit32 for specified file
//
// Author:     Peter Sawatzki (ps)
//             Buchenhof 3, 58091 Hagen, Germany
// CompuServe: 100031,3002
// E-Mail:     Peter@Sawatzki.de
// web:        http://www.sawatzki.de
//
// Date:    Author:
// 07/21/97 ps    first version
// 02/05/98 ps    fixed () bug
// 28/11/00 ps    update for Delphi5
//
// Copyright  1996-2000 Peter Sawatzki. All Rights Reserved.

Unit LaunchM;
Interface
Uses
  Windows, ActiveX, ComObj, ShlObj;

Const
  Class_ContextMenu: TGUID = '{8D1B3EF0-1C73-11D1-99B1-0000929B06C1}';
type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    szFile: array[0..MAX_PATH] of Char;
  public
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

Implementation
Uses
  ComServ, SysUtils, ShellApi, Registry, IniFiles;
Const
  KEditModuleDefault = 'KEditW32.Exe';
  KEditIni = 'KEditW.Ini';
  LaunchHelpText = 'Edit file with KEdit32';
  LaunchKey = '*\shellex\ContextMenuHandlers\KEdit launch extension';
Var
  KEditModule: String = '';

Function GetKEditPath: string;
// Returns string containing path KEdit
begin
  If KEditModule='' Then Begin
    KEditModule:= KEditModuleDefault;
    With TIniFile.Create(KEditIni) Do
    Try
      KEditModule:= ReadString('State32', 'ModuleName', KEditModuleDefault)
    Finally
      Free
    End
  End;
  Result:= KEditModule+' "%s"'
end;

function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
begin
  Result:= 0;
  If ((uFlags And $0000000F) = CMF_NORMAL) Or ((uFlags And CMF_EXPLORE) <> 0) Then Begin
    // Add one menu item to context menu
    InsertMenu (Menu, indexMenu, MF_STRING Or MF_BYPOSITION, idCmdFirst, '&KEdit');
    // Return number of menu items added
    Result:= 1
  End
End;

Function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
Var
  SI: TStartupInfo;
  PI: TProcessInformation;
  TheDir: String;
Begin
  // Make sure we are not being called by an application
  If HiWord(Integer(lpici.lpVerb)) <> 0 then begin
    Result := E_FAIL;
    Exit
  End;
  // Make sure we aren't being passed an invalid argument number
  If LoWord(lpici.lpVerb) > 0 then begin
    Result:= E_INVALIDARG;
    Exit
  End;
  // Execute the command specified by lpici.lpVerb.
  If LoWord(lpici.lpVerb)=0 Then Begin
    // invoke KEdit
    FillChar(SI, SizeOf(SI), 0);
    FillChar(PI, SizeOf(PI), 0);
    SI.cb:= SizeOf(SI);
    TheDir:= ExtractFileDir(szFile);
    CreateProcess(Nil, PChar(Format(GetKEditPath, [szFile])), Nil, Nil,
      False, 0, Nil, PChar(TheDir), SI, PI);
    CloseHandle(PI.hThread);
    CloseHandle(PI.hProcess);
  End;
  Result := NOERROR
End;

function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin
  if idCmd = 0 then begin
    // return help string for menu item
    If uType=GCS_HELPTEXTW Then
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, LaunchHelpText, -1, PWideChar(pszName), cchMax)
    Else
      StrCopy(pszName, LaunchHelpText);
    Result := NOERROR;
  end else
    Result := E_INVALIDARG;
end;

function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  medium: TStgMedium;
  fe: TFormatEtc;
begin
  // Fail the call if lpdobj is Nil.
  if (lpdobj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with fe do begin
    cfFormat := CF_HDROP;
    ptd := Nil;
    dwAspect := DVASPECT_CONTENT;
    lindex := -1;
    tymed := TYMED_HGLOBAL;
  end;
  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(fe, medium);
  if Failed(Result) then Exit;
  // If only one file is selected, retrieve the file name and store it in
  // szFile. Otherwise fail the call.
  if DragQueryFile(medium.hGlobal, $FFFFFFFF, Nil, 0) = 1 then begin
    DragQueryFile(medium.hGlobal, 0, szFile, SizeOf(szFile));
    Result := NOERROR;
  end else Begin
    szFile[0]:= #0;
    Result := E_FAIL
  End;
  ReleaseStgMedium(medium)
end;

Type
  TContextMenuFactory = Class(TComObjectFactory)
  Public
    procedure UpdateRegistry(Register: Boolean); override;
  End;

Procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
Var
  ClassID: String;
Begin
  If Register Then Begin
    Inherited UpdateRegistry(Register);

    ClassID := GUIDToString(Class_ContextMenu);
    CreateRegKey(LaunchKey, '', ClassID);

    if (Win32Platform = VER_PLATFORM_WIN32_NT) then
      with TRegistry.Create do
        try
          RootKey := HKEY_LOCAL_MACHINE;
          OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
          OpenKey('Approved', True);
          WriteString(ClassID, LaunchHelpText);
        finally
          Free;
        end;
  end else begin
    DeleteRegKey(LaunchKey);
    inherited UpdateRegistry(Register)
  end;
end;

initialization
  TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu,
    '', 'KEdit launcher', ciMultiInstance, tmApartment);
end.
